<?php

/*
 * This file is part of Monsieur Biz' Search plugin for Sylius.
 *
 * (c) Monsieur Biz <sylius@monsieurbiz.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

declare(strict_types=1);

namespace MonsieurBiz\SyliusSearchPlugin\Helper;

class SortHelper implements SortHelperInterface
{
    public function getSortParamByField(string $field, string $channel, string $order = 'asc', string $taxon = ''): array
    {
        switch ($field) {
            case 'name':
            case 'created_at':
                return $this->buildSort('attributes.value.keyword', $order, 'attributes', 'attributes.code', $field);
            case 'price':
                return $this->buildSort('price.value', $order, 'price', 'price.channel', $channel);
            case 'position':
                return $this->buildSort('taxon.productPosition', $order, 'taxon', 'taxon.code', $taxon);
            default:
                // Dummy value to have null sorting in ES and keep ES results sorting
                return $this->buildSort('attributes.value.keyword', $order, 'attributes', 'attributes.code', 'dummy');
        }
    }

    public function buildSort(
        string $field,
        string $order,
        string $nestedPath,
        string $sortFilterField,
        string $sortFilterValue
    ): array {
        return [
            $field => [
                'order' => $order,
                'nested' => [
                    'path' => $nestedPath,
                    'filter' => [
                        'term' => [$sortFilterField => $sortFilterValue],
                    ],
                ],
            ],
        ];
    }
}
